@@ -16,6 +16,7 @@
 #include "style.h"
 #include "ms-excel-util.h"
 #include <goffice/goffice.h>
+#include <glib/gi18n-lib.h>
 
 #include <string.h>
 
@@ -683,3 +684,136 @@ xls_paper_size (GtkPaperSize *ps, gboolean rotated)
 
 	return 0;
 }
+
+
+static void
+xls_header_footer_export1 (GString *res, const char *s, const char *section)
+{
+	static const struct {
+		const char *name;
+		const char *xls_code;
+	} codes[] = {
+		{ N_("TAB"),   "&A"},
+		{ N_("PAGE"),  "&P"},
+		{ N_("PAGES"), "&N"},
+		{ N_("DATE"),  "&D"},
+		{ N_("TIME"),  "&T"},
+		{ N_("FILE"),  "&F"},
+		{ N_("PATH"),  "&Z"},
+#if 0
+		{ N_("CELL"),  "" /* ??? */},
+		{ N_("TITLE"), "" /* ??? */}
+#endif
+	};
+
+	if (!s || *s == 0)
+		return;
+
+	g_string_append (res, section);
+	while (*s) {
+		const char *end;
+
+		if (*s == '&' && s[1] == '[' && (end = strchr (s + 2, ']'))) {
+			size_t l = end - (s + 2);
+			unsigned ui;
+
+			for (ui = 0; ui < G_N_ELEMENTS (codes); ui++) {
+				const char *tname = _(codes[ui].name);
+				if (l == strlen (tname) &&
+				    g_ascii_strncasecmp (tname, s + 2, l) == 0) {
+					g_string_append (res, codes[ui].xls_code);
+					break;
+				}
+			}
+			s = end + 1;
+			continue;
+		}
+
+		g_string_append_c (res, *s++);
+	}
+}
+
+
+char *
+xls_header_footer_export (const PrintHF *hf)
+{
+	GString *res = g_string_new (NULL);
+
+	xls_header_footer_export1 (res, hf->left_format, "&L");
+	xls_header_footer_export1 (res, hf->middle_format, "&C");
+	xls_header_footer_export1 (res, hf->right_format, "&R");
+
+	return g_string_free (res, FALSE);
+}
+
+void
+xls_header_footer_import (PrintHF *hf, const char *txt)
+{
+	char section = 'L';
+	GString *accum = g_string_new (NULL);
+
+	g_free (hf->left_format); hf->left_format = g_strdup ("");
+	g_free (hf->middle_format); hf->middle_format = g_strdup ("");
+	g_free (hf->right_format); hf->right_format = g_strdup ("");
+
+	while (1) {
+		if (txt[0] == 0 ||
+		    (txt[0] == '&' && strchr ("LCR", txt[1]))) {
+			char **sp;
+			switch (section) {
+			case 'L': sp = &hf->left_format; break;
+			case 'C': sp = &hf->middle_format; break;
+			case 'R': sp = &hf->right_format; break;
+			default: g_assert_not_reached ();
+			}
+			g_free (*sp);
+			*sp = g_string_free (accum, FALSE);
+
+			if (txt[0] == 0)
+				break;
+
+			accum = g_string_new (NULL);
+			section = txt[1];
+			txt += 2;
+			continue;
+		}
+
+		if (txt[0] != '&') {
+			g_string_append_c (accum, *txt++);
+			continue;
+		}
+
+		txt++;
+		switch (txt[0]) {
+		case 0:
+			continue;
+		case '&':
+			g_string_append_c (accum, *txt);
+			break;
+		case 'A':
+			g_string_append (accum, "&[TAB]");
+			break;
+		case 'P':
+			g_string_append (accum, "&[PAGE]");
+			break;
+		case 'N':
+			g_string_append (accum, "&[PAGES]");
+			break;
+		case 'D':
+			g_string_append (accum, "&[DATE]");
+			break;
+		case 'T':
+			g_string_append (accum, "&[TIME]");
+			break;
+		case 'F':
+			g_string_append (accum, "&[FILE]");
+			break;
+		case 'Z':
+			g_string_append (accum, "&[PATH]");
+			break;
+		default:
+			break;
+		}
+		txt++;
+	}
+}
