@@ -357,12 +357,50 @@ write_cellref_v8 (PolishData *pd, GnmCellRef const *ref,
 }
 
 static void
-write_string (PolishData *pd, gchar const *txt)
+write_string1 (PolishData *pd, gchar const *txt)
 {
 	push_guint8 (pd, FORMULA_PTG_STR);
 	excel_write_string (pd->ewb->bp, STR_ONE_BYTE_LENGTH, txt);
 }
 
+static void
+write_string (PolishData *pd, gchar const *txt)
+{
+	size_t i, n = 0, len = g_utf8_strlen (txt, -1);
+	const char *p = txt;
+	const size_t CHUNK_LEN = 255;
+
+	/*
+	 * The xls format only handles strings up to 255 characters
+	 * in formulae.  To make up for that, we create expressions
+	 * like ("s1"&"s2"&...).  Note, that the parentheses are
+	 * explicitly present in the file.
+	 */
+	for (i = 0; n == 0 || i < len; ) {
+		if (len - i <= CHUNK_LEN) {
+			write_string1 (pd, p);
+			i = len;
+		} else {
+			const char *endcut = g_utf8_offset_to_pointer (p, CHUNK_LEN);
+			size_t cutlen = endcut - p;
+			char *cut = g_memdup (p, cutlen + 1);
+			cut[cutlen] = 0;
+			write_string1 (pd, cut);
+			g_free (cut);
+
+			p = endcut;
+			i += CHUNK_LEN;
+		}
+
+		if (n > 0)
+			push_guint8 (pd, FORMULA_PTG_CONCAT);
+		n++;
+	}
+
+	if (n > 1)
+		push_guint8 (pd, FORMULA_PTG_PAREN);
+}
+
 static void
 excel_formula_write_CELLREF (PolishData *pd, GnmCellRef const *ref,
 			     Sheet *sheet_b, XLOpType target_type)
@@ -992,9 +1030,9 @@ excel_write_array_formula (ExcelWriteState *ewb,
 	pd.use_name_variant = FALSE;
 	pd.allow_sheetless_ref = TRUE;
 
-	start = ewb->bp->length;
+	start = ewb->bp->curpos;
 	write_node (&pd, array->expr, 0, XL_ROOT);
-	len = ewb->bp->length - start;
+	len = ewb->bp->curpos - start;
 
 	write_arrays (&pd);
 
