@@ -185,7 +185,7 @@ xlsx_add_pt (GsfXMLOut *xml, char const *id, double l)
 
 #define N_PREDEFINED_FILLS (G_N_ELEMENTS (pre_def_fills) - 1)
 
-static char const *pats[] = {
+static char const * const pats[] = {
 	"solid",            /* 1 */
 	"darkGray",         /* 2 */
 	"mediumGray",       /* 3 */
@@ -204,9 +204,16 @@ static char const *pats[] = {
 	"lightUp",          /* 16 */
 	"lightGrid",        /* 17 */
 	"lightTrellis",     /* 18 */
+	/* Not in Excel */
+	"lightVertical",    /* 19 */ 
+	"darkHorizontal",   /* 20 */
+	"lightGray",        /* 21 */
+	"lightGray",        /* 22 */
+	"darkGray",         /* 23 */
+	"solid",            /* 24 */
 };
 
-static char const *pre_def_fills[] = {
+static char const * const pre_def_fills[] = {
 	"none",
 	"gray125",
 	NULL
@@ -215,7 +222,7 @@ static char const *pre_def_fills[] = {
 static void
 xlsx_write_predefined_fills (GsfXMLOut *xml)
 {
-	char const **f = pre_def_fills;
+	char const * const *f = pre_def_fills;
 
 	while (*f != NULL) {
 		gsf_xml_out_start_element (xml, "fill");
@@ -418,6 +425,8 @@ xlsx_find_font (GnmStyle const *style, GPtrArray  *styles)
 		    gnm_style_is_element_set (old_style, MSTYLE_FONT_BOLD) ||
 		    gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC) !=
 		    gnm_style_is_element_set (old_style, MSTYLE_FONT_ITALIC) ||
+		    gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) !=
+		    gnm_style_is_element_set (old_style, MSTYLE_FONT_UNDERLINE) ||
 		    gnm_style_is_element_set (style, MSTYLE_FONT_COLOR) !=
 		    gnm_style_is_element_set (old_style, MSTYLE_FONT_COLOR) ||
 		    gnm_style_is_element_set (style, MSTYLE_FONT_NAME) !=
@@ -429,6 +438,7 @@ xlsx_find_font (GnmStyle const *style, GPtrArray  *styles)
 			continue;
 		if (gnm_style_get_font_bold (style) != gnm_style_get_font_bold (old_style) ||
 		    gnm_style_get_font_italic (style) != gnm_style_get_font_italic (old_style) ||
+		    gnm_style_get_font_uline (style) != gnm_style_get_font_uline (old_style) ||
 		    gnm_style_get_font_color (style)->go_color !=
 		    gnm_style_get_font_color (old_style)->go_color ||
 		    gnm_style_get_font_size (style) != gnm_style_get_font_size (old_style) ||
@@ -452,6 +462,7 @@ xlsx_write_fonts (XLSXWriteState *state, GsfXMLOut *xml)
 		GnmStyle const *style = g_ptr_array_index (state->styles_array, i);
 		if (gnm_style_is_element_set (style, MSTYLE_FONT_BOLD) ||
 		    gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC) ||
+		    gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
 		    gnm_style_is_element_set (style, MSTYLE_FONT_COLOR) ||
 		    gnm_style_is_element_set (style, MSTYLE_FONT_NAME) ||
 		    gnm_style_is_element_set (style, MSTYLE_FONT_SIZE) ||
@@ -483,6 +494,20 @@ xlsx_write_fonts (XLSXWriteState *state, GsfXMLOut *xml)
 				xlsx_add_bool (xml, "val", gnm_style_get_font_italic (style));
 				gsf_xml_out_end_element (xml);
 			}
+			if (gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE)) {
+				static const char * const types[] = {
+					"none",
+					"single",
+					"double",
+					"singleAccounting",
+					"doubleAccounting"
+				};
+
+				gsf_xml_out_start_element (xml, "u");
+				gsf_xml_out_add_cstr
+					(xml, "val", types[gnm_style_get_font_uline (style)]);
+				gsf_xml_out_end_element (xml);
+			}
 			if (gnm_style_is_element_set (style, MSTYLE_FONT_COLOR))
 				xlsx_write_color_element
 					(xml, "color",
@@ -516,7 +541,7 @@ xlsx_write_fonts (XLSXWriteState *state, GsfXMLOut *xml)
 }
 
 static gint
-xlsx_find_fill (GnmStyle const *style, GPtrArray  *styles)
+xlsx_find_fill (GnmStyle const *style, GPtrArray *styles)
 {
 	unsigned i;
 	int j;
@@ -927,7 +952,12 @@ xlsx_write_style_write_alignment (G_GNUC_UNUSED XLSXWriteState *state, GsfXMLOut
 		gsf_xml_out_add_bool (xml, "shrinkToFit", gnm_style_get_shrink_to_fit (style));
 	}
 	if (gnm_style_is_element_set (style, MSTYLE_ROTATION)) {
-		gsf_xml_out_add_int (xml, "textRotation", gnm_style_get_rotation (style));
+		int r = gnm_style_get_rotation (style);
+		if (r == -1)
+			r = 0xff;
+		else if (r >= 270)
+			r = (360 - r) + 90;
+		gsf_xml_out_add_int (xml, "textRotation", r);
 	}
 	if (gnm_style_is_element_set (style, MSTYLE_INDENT)) {
 		gsf_xml_out_add_int (xml, "indent", gnm_style_get_indent (style));
@@ -1186,6 +1216,7 @@ row_boring (Sheet *sheet, int r)
 		return TRUE;
 
 	return (!ri->hard_size &&
+		fabs (ri->size_pts - sheet->rows.default_style.size_pts) < 1e-6 &&
 		!ri->is_collapsed &&
 		ri->visible &&
 		ri->outline_level == 0);
@@ -1233,9 +1264,12 @@ xlsx_write_cells (XLSXWriteState *state, GsfXMLOut *xml,
 
 			if (ri->hard_size) {
 				xlsx_write_init_row (&needs_row, xml, r, cheesy_span);
-				gsf_xml_out_add_float (xml, "ht", ri->size_pts, 4);
 				gsf_xml_out_add_cstr_unchecked (xml, "customHeight", "1");
 			}
+			if (ri->hard_size || fabs (ri->size_pts - sheet->cols.default_style.size_pts) > 1e-6) {
+				xlsx_write_init_row (&needs_row, xml, r, cheesy_span);
+				gsf_xml_out_add_float (xml, "ht", ri->size_pts, 4);
+			}
 			if (ri->is_collapsed) {
 				xlsx_write_init_row (&needs_row, xml, r, cheesy_span);
 				gsf_xml_out_add_cstr_unchecked (xml, "collapsed", "1");
@@ -1649,42 +1683,89 @@ static void
 xlsx_write_autofilters (XLSXWriteState *state, GsfXMLOut *xml)
 {
 	GnmFilter const *filter;
-	GnmFilterCondition const *cond;
 	unsigned i;
 
 	if (NULL == state->sheet->filters)
 		return;
 
+	/* We write only the first filter per sheet.  */
 	filter = state->sheet->filters->data;
+
 	gsf_xml_out_start_element (xml, "autoFilter");
 	xlsx_add_range (xml, "ref", &filter->r);
 
 	for (i = 0; i < filter->fields->len ; i++) {
+		GnmFilterCondition const *cond =
+			gnm_filter_get_condition (filter, i);
+
 		/* filter unused or bucket filters in excel5 */
-		if (NULL == (cond = gnm_filter_get_condition (filter, i)) ||
-		    cond->op[0] == GNM_FILTER_UNUSED)
+		if (!cond || cond->op[0] == GNM_FILTER_UNUSED)
 			continue;
 
 		gsf_xml_out_start_element (xml, "filterColumn");
 		gsf_xml_out_add_int (xml, "colId", i);
 
 		switch (cond->op[0]) {
-		case GNM_FILTER_OP_EQUAL :
-		case GNM_FILTER_OP_GT :
-		case GNM_FILTER_OP_LT :
-		case GNM_FILTER_OP_GTE :
-		case GNM_FILTER_OP_LTE :
-		case GNM_FILTER_OP_NOT_EQUAL :
+		case GNM_FILTER_OP_EQUAL:
+		case GNM_FILTER_OP_GT:
+		case GNM_FILTER_OP_LT:
+		case GNM_FILTER_OP_GTE:
+		case GNM_FILTER_OP_LTE:
+		case GNM_FILTER_OP_NOT_EQUAL: {
+			static const char *const opname[6] = {
+				"equal", "greaterThan", "lessThan",
+				"greaterThanOrEqual", "lessThanOrEqual",
+				"notEqual"
+			};
+			unsigned oi, N;
+
+			for (oi = N = 1; oi < G_N_ELEMENTS (cond->op); oi++) {
+				if (cond->op[oi] == GNM_FILTER_UNUSED)
+					continue;
+				N++;
+			}
+
+			gsf_xml_out_start_element (xml, "customFilters");
+			if (N > 1)
+				gsf_xml_out_add_cstr_unchecked (xml, "and", "true");
+			for (oi = 0; oi < G_N_ELEMENTS (cond->op); oi++) {
+				GnmFilterOp op = cond->op[oi];
+				GString *str;
+
+				if (op == GNM_FILTER_UNUSED)
+					continue;
+				gsf_xml_out_start_element (xml, "customFilter");
+				if (op >= GNM_FILTER_OP_EQUAL && op <= GNM_FILTER_OP_NOT_EQUAL)
+					gsf_xml_out_add_cstr_unchecked (xml, "operator", opname[op]);
+
+				str = g_string_new (NULL);
+				value_get_as_gstring (cond->value[oi], str, state->convs);
+				gsf_xml_out_add_cstr (xml, "val", str->str);
+				g_string_free (str, TRUE);
+				gsf_xml_out_end_element (xml); /* </customFilter> */
+			}
+			gsf_xml_out_end_element (xml); /* </customFilters> */
 			break;
+		}
 
 		case GNM_FILTER_OP_BLANKS :
+			gsf_xml_out_start_element (xml, "filters");
+			xlsx_add_bool (xml, "blank", TRUE);
+			gsf_xml_out_end_element (xml); /* </filters> */
+			break;
 		case GNM_FILTER_OP_NON_BLANKS :
+			gsf_xml_out_start_element (xml, "customFilters");
+			gsf_xml_out_start_element (xml, "customFilter");
+			gsf_xml_out_add_cstr_unchecked (xml, "operator", "notEqual");
+			gsf_xml_out_add_cstr (xml, "val", " ");
+			gsf_xml_out_end_element (xml); /* </customFilter> */
+			gsf_xml_out_end_element (xml); /* </customFilters> */
 			break;
 
-		case GNM_FILTER_OP_TOP_N :
-		case GNM_FILTER_OP_BOTTOM_N :
-		case GNM_FILTER_OP_TOP_N_PERCENT :
-		case GNM_FILTER_OP_BOTTOM_N_PERCENT :
+		case GNM_FILTER_OP_TOP_N:
+		case GNM_FILTER_OP_BOTTOM_N:
+		case GNM_FILTER_OP_TOP_N_PERCENT:
+		case GNM_FILTER_OP_BOTTOM_N_PERCENT:
 			gsf_xml_out_start_element (xml, "top10");
 			gsf_xml_out_add_float (xml, "val", cond->count, -1);
 			if (cond->op[0] & GNM_FILTER_OP_BOTTOM_MASK)
@@ -1694,7 +1775,7 @@ xlsx_write_autofilters (XLSXWriteState *state, GsfXMLOut *xml)
 			gsf_xml_out_end_element (xml); /* </top10> */
 			break;
 
-		default :
+		default:
 			continue;
 		}
 
@@ -1951,6 +2032,19 @@ xlsx_find_paper_code (GtkPaperSize *psize)
 	return 0;
 }
 
+static void
+xlsx_write_print_info_hf (XLSXWriteState *state, GsfXMLOut *xml,
+			  const PrintHF *hf, const char *hftext)
+{
+	char *s = xls_header_footer_export (hf);
+
+	gsf_xml_out_start_element (xml, hftext);
+	gsf_xml_out_add_cstr (xml, NULL, s);
+	gsf_xml_out_end_element (xml); /* hftext */
+
+	g_free (s);
+}
+
 static void
 xlsx_write_print_info (XLSXWriteState *state, GsfXMLOut *xml)
 {
@@ -2069,16 +2163,15 @@ xlsx_write_print_info (XLSXWriteState *state, GsfXMLOut *xml)
 
 	gsf_xml_out_end_element (xml); /* </pageSetup> */
 
+	gsf_xml_out_start_element (xml, "headerFooter");
+	xlsx_write_print_info_hf (state, xml, pi->header, "oddHeader");
+	xlsx_write_print_info_hf (state, xml, pi->footer, "oddFooter");
+	gsf_xml_out_end_element (xml); /* </headerFooter> */
+
 	if (NULL != pi->page_breaks.v)
 		xlsx_write_breaks (state, xml, pi->page_breaks.v);
 	if (NULL != pi->page_breaks.h)
 		xlsx_write_breaks (state, xml, pi->page_breaks.h);
-
-#if 0
-	gsf_xml_out_start_element (xml, "headerFooter");
-	gsf_xml_out_end_element (xml); /* </headerFooter> */
-#endif
-
 }
 
 /**********************************************************************/
@@ -2506,9 +2599,14 @@ xlsx_write_workbook (XLSXWriteState *state, GsfOutfile *root_part)
 
 	gsf_xml_out_start_element (xml, "bookViews");
 	WORKBOOK_FOREACH_VIEW (state->base.wb, view, {
+		int scale = 10;  /* Guess */
 		gsf_xml_out_start_element (xml, "workbookView");
 		gsf_xml_out_add_int (xml, "activeTab",
 			view->current_sheet->index_in_wb);
+		if (view->preferred_width > 0)
+			gsf_xml_out_add_int (xml, "windowWidth", view->preferred_width * scale);
+		if (view->preferred_height > 0)
+			gsf_xml_out_add_int (xml, "windowHeight", view->preferred_height * scale);
 		gsf_xml_out_end_element (xml);
 	});
 	gsf_xml_out_end_element (xml);
