@@ -56,6 +56,7 @@
 #include <parse-util.h>
 #include <rendered-value.h>
 #include <cellspan.h>
+#include <print-info.h>
 
 #include <locale.h>
 #include <gsf/gsf-output.h>
@@ -476,6 +477,13 @@ latex2e_write_font_encodings (GsfOutput *output, Sheet *sheet, GnmRange const *r
 static void
 latex2e_write_file_header(GsfOutput *output, Sheet *sheet, GnmRange const *range)
 {
+	gboolean is_landscape = FALSE, use_utf8;
+	GtkPageOrientation orient = print_info_get_paper_orientation (sheet->print_info);
+
+	is_landscape = (orient == GTK_PAGE_ORIENTATION_LANDSCAPE ||
+			orient == GTK_PAGE_ORIENTATION_REVERSE_LANDSCAPE);
+	use_utf8 = gnm_conf_get_plugin_latex_use_utf8 ();
+
 	gsf_output_puts (output,
 "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
 "%%                                                                  %%\n"
@@ -496,7 +504,7 @@ latex2e_write_file_header(GsfOutput *output, Sheet *sheet, GnmRange const *range
 "%%  rendered correctly:                                             %%\n"
 );
 
-	if (gnm_conf_get_plugin_latex_use_utf8 ())
+	if (use_utf8)
 		gsf_output_puts (output,
 "%%    \\usepackage{ucs}                                              %%\n"
 "%%    \\usepackage[utf8x]{inputenc}                                  %%\n"
@@ -540,15 +548,24 @@ latex2e_write_file_header(GsfOutput *output, Sheet *sheet, GnmRange const *range
 "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
 "%%                                                                  %%\n"
 "%%  This is the PREAMBLE. Change these values to get the right      %%\n"
-"%%  paper size and other niceties. Uncomment the landscape option   %%\n"
-"%%  to the documentclass defintion for standalone documents.        %%\n"
+"%%  paper size and other niceties.                                  %%\n"
 "%%                                                                  %%\n"
 "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
 "\n"
+		);
+	if (is_landscape)
+		gsf_output_puts (output,
+"	\\documentclass[12pt%\n"
+"			  ,landscape%\n"
+"                    ]{report}\n"
+				 );
+	else
+		gsf_output_puts (output,
 "	\\documentclass[12pt%\n"
 "			  %,landscape%\n"
 "                    ]{report}\n"
-		);
+				 );
+
 
 
 	if (gnm_conf_get_plugin_latex_use_utf8 ()) {
@@ -700,7 +717,7 @@ latex2e_write_table_header(GsfOutput *output, int num_cols)
  *
  */
 static GnmStyleBorderType
-latex2e_find_vline (int col, int row, Sheet *sheet, GnmStyleElement which_border)
+latex2e_find_this_vline (int col, int row, Sheet *sheet, GnmStyleElement which_border)
 {
 	GnmBorder const	*border;
 	GnmStyle const	*style;
@@ -711,8 +728,7 @@ latex2e_find_vline (int col, int row, Sheet *sheet, GnmStyleElement which_border
 	style = sheet_style_get (sheet, col, row);
 	border = gnm_style_get_border (style, which_border);
 
-	if (!(gnm_style_border_is_blank (border) ||
-	      border->line_type == GNM_STYLE_BORDER_NONE))
+	if (!gnm_style_border_is_blank (border))
 		return border->line_type;
 
 	if (which_border == MSTYLE_BORDER_LEFT) {
@@ -720,20 +736,43 @@ latex2e_find_vline (int col, int row, Sheet *sheet, GnmStyleElement which_border
 			return GNM_STYLE_BORDER_NONE;
 		style = sheet_style_get (sheet, col - 1, row);
 		border = gnm_style_get_border (style, MSTYLE_BORDER_RIGHT);
-		return ((gnm_style_border_is_blank (border)) ? GNM_STYLE_BORDER_NONE :
-			border->line_type);
+		return ((border == NULL) ? GNM_STYLE_BORDER_NONE : border->line_type);
 	} else {
 		if ((col+1) >= colrow_max (TRUE, sheet))
 		    return GNM_STYLE_BORDER_NONE;
 		style = sheet_style_get (sheet, col + 1, row);
 		border = gnm_style_get_border (style, MSTYLE_BORDER_LEFT);
-		return ((gnm_style_border_is_blank (border)) ? GNM_STYLE_BORDER_NONE :
-			border->line_type);
+		return ((border == NULL) ? GNM_STYLE_BORDER_NONE : border->line_type);
 	}
 
 	return GNM_STYLE_BORDER_NONE;
 }
 
+static GnmStyleBorderType
+latex2e_find_vline (int col, int row, Sheet *sheet, GnmStyleElement which_border)
+{
+	/* We are checking for NONE boreders first since there should only be a few merged ranges */
+	GnmStyleBorderType result = latex2e_find_this_vline (col, row, sheet, which_border);
+	GnmCellPos pos;
+	GnmRange const * range;
+
+	if (result == GNM_STYLE_BORDER_NONE)
+		return GNM_STYLE_BORDER_NONE;
+	
+	pos.col = col;
+	pos.row = row;
+	range = gnm_sheet_merge_contains_pos (sheet, &pos);
+
+	if (range) {
+		if ((which_border == MSTYLE_BORDER_LEFT && col == range->start.col)
+		    || (which_border == MSTYLE_BORDER_RIGHT&& col == range->end.col))
+			return result;
+		else
+			return GNM_STYLE_BORDER_NONE;	
+	}
+	return result;
+}
+
 /**
  * latex2e_print_vert_border:
  *
@@ -1136,8 +1175,8 @@ latex2e_find_hhlines (GnmStyleBorderType *clines, G_GNUC_UNUSED int length, int
 {
 	GnmStyle const	*style;
 	GnmBorder const	*border;
-	/* GnmRange const	*merge_range; */
-	/* GnmCellPos pos; */
+	GnmRange const	*range;
+	GnmCellPos pos;
 
 	style = sheet_style_get (sheet, col, row);
 	border = gnm_style_get_border (style, type);
@@ -1145,19 +1184,17 @@ latex2e_find_hhlines (GnmStyleBorderType *clines, G_GNUC_UNUSED int length, int
 		return FALSE;
 	clines[0] = border->line_type;
 
-	/* The following code completes the border above a merged cell. As long as we allow partial */
-	/* borders above & below merged cells in Gnumeric we should not enable this.                */ 
-	/* pos.col = col; */
-	/* pos.row = row; */
-	/* merge_range = gnm_sheet_merge_is_corner (sheet, &pos); */
-	/* if (merge_range != NULL) { */
-	/* 	int i; */
-
-	/* 	for (i = 1; i < MIN (merge_range->end.col - merge_range->start.col + 1, */
-	/* 			    length); i++) */
-	/* 		     clines[i] = border->line_type; */
-	/* } */
-
+	pos.col = col;
+	pos.row = row;
+	range = gnm_sheet_merge_contains_pos (sheet, &pos);
+	if (range) {
+		if ((type == MSTYLE_BORDER_TOP && row > range->start.row)
+		    || (type == MSTYLE_BORDER_BOTTOM&& row < range->end.row)) {
+			clines[0] = GNM_STYLE_BORDER_NONE;
+			return FALSE;
+		}
+	}
+	
 	return TRUE;
 }
 
@@ -1235,7 +1272,7 @@ file_saver_sheet_get_extent (Sheet *sheet)
 		if (start_sheet == sheet)
 			return r;
 	}
-	return sheet_get_extent (sheet, TRUE);
+	return sheet_get_extent (sheet, TRUE, TRUE);
 }
 
 /**
