@@ -211,6 +211,8 @@ item_grid_realize (GocItem *item)
 			gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "cursor_cross", 32, 0, NULL),
 			17, 17);
 	cb_cursor_motion (ig);
+	/* we need to destroy the hlink tip when the state flags change */
+	g_signal_connect_swapped (item->canvas, "state-flags-changed", G_CALLBACK (ig_clear_hlink_tip), item);
 }
 
 static void
@@ -514,11 +516,18 @@ item_grid_draw_region (GocItem const *item, cairo_t *cr,
 
 	/* Fill entire region with default background (even past far edge) */
 	cairo_save (cr);
-	gtk_render_background (goc_item_get_style_context (item),
-			       cr,
-			       x0 - canvas->scroll_x1 * scale,
-			       y0 - canvas->scroll_y1 * scale,
-			       width, height);
+	if (canvas->direction == GOC_DIRECTION_LTR)
+		gtk_render_background (goc_item_get_style_context (item),
+				       cr,
+				       x0 - canvas->scroll_x1 * scale,
+				       y0 - canvas->scroll_y1 * scale,
+				       width, height);
+	else
+		gtk_render_background (goc_item_get_style_context (item),
+				       cr,
+				       canvas->width - x0 + canvas->scroll_x1 * scale - width,
+				       y0 - canvas->scroll_y1 * scale,
+				       width, height);
 	cairo_restore (cr);
 
 	/* Get ordered list of merged regions */
