@@ -189,7 +189,9 @@ gnm_pre_parse_init (int argc, gchar const **argv)
 #if !GLIB_CHECK_VERSION(2,32,0)
 	g_thread_init (NULL);
 #endif
+#if !GLIB_CHECK_VERSION(2,36,0)
 	g_type_init ();
+#endif
 
 	/* On win32 argv contains 'ansi' encoded args.  We need to manually
 	 * pull in the real versions and convert them to utf-8 */
@@ -220,20 +222,6 @@ gnm_pre_parse_shutdown (void)
 	go_shell_argv_to_glib_encoding_free ();
 }
 
-#if 0
-static void
-gnumeric_check_for_components (void)
-{
-	OAF_ServerInfoList *result;
-
-	result = oaf_query ("repo_ids.has('IDL::1.0')", NULL, NULL);
-
-	g_return_if_fail (result != NULL);
-
-	CORBA_free (info_list);
-}
-#endif
-
 void
 gnm_init (void)
 {
@@ -272,12 +260,7 @@ gnm_init (void)
 	print_init ();
 	gnm_autofill_init ();
 	sheet_objects_init ();
-
-	/* make sure that all hlink types are registered */
-	gnm_hlink_cur_wb_get_type ();
-	gnm_hlink_url_get_type ();
-	gnm_hlink_email_get_type ();
-	gnm_hlink_external_get_type ();
+	_gnm_hlink_init ();
 
 	/* The statically linked in file formats */
 	gnm_xml_sax_read_init ();
