@@ -27,6 +27,7 @@ G_BEGIN_DECLS
 
 /* ------------------------------------------------------------------------- */
 
+gnm_float gnm_log2 (gnm_float x);
 gnm_float log1pmx (gnm_float x);
 gnm_float swap_log_tail (gnm_float lp);
 gnm_float pow1p (gnm_float x, gnm_float y);
@@ -36,22 +37,16 @@ gnm_float logspace_add (gnm_float logx, gnm_float logy);
 gnm_float logspace_sub (gnm_float logx, gnm_float logy);
 gnm_float gnm_owent (gnm_float h, gnm_float a);
 gnm_float gnm_logcf (gnm_float x, gnm_float i, gnm_float d);
+gnm_float expmx2h (gnm_float x);
 
 /* "d": density.  */
 /* "p": distribution function.  */
 /* "q": inverse distribution function.  */
 
 /* The normal distribution.  */
-gnm_float dnorm (gnm_float x, gnm_float mu, gnm_float sigma, gboolean give_log);
 gnm_float pnorm (gnm_float x, gnm_float mu, gnm_float sigma, gboolean lower_tail, gboolean log_p);
-gnm_float pnorm2 (gnm_float x1, gnm_float x2);
 gnm_float qnorm (gnm_float p, gnm_float mu, gnm_float sigma, gboolean lower_tail, gboolean log_p);
 
-/* The log-normal distribution.  */
-gnm_float dlnorm (gnm_float x, gnm_float logmean, gnm_float logsd, gboolean give_log);
-gnm_float plnorm (gnm_float x, gnm_float logmean, gnm_float logsd, gboolean lower_tail, gboolean log_p);
-gnm_float qlnorm (gnm_float p, gnm_float logmean, gnm_float logsd, gboolean lower_tail, gboolean log_p);
-
 /* The gamma distribution.  */
 gnm_float dgamma (gnm_float x, gnm_float shape, gnm_float scale, gboolean give_log);
 gnm_float pgamma (gnm_float x, gnm_float shape, gnm_float scale, gboolean lower_tail, gboolean log_p);
@@ -128,21 +123,6 @@ gnm_float qtukey(gnm_float p, gnm_float nmeans, gnm_float df, gnm_float nranges,
 
 /* ------------------------------------------------------------------------- */
 
-typedef gnm_float (*GnmPFunc) (gnm_float x, const gnm_float shape[],
-			       gboolean lower_tail, gboolean log_p);
-typedef gnm_float (*GnmDPFunc) (gnm_float x, const gnm_float shape[],
-				gboolean log_p);
-
-gnm_float pfuncinverter (gnm_float p, const gnm_float shape[],
-			 gboolean lower_tail, gboolean log_p,
-			 gnm_float xlow, gnm_float xhigh, gnm_float x0,
-			 GnmPFunc pfunc, GnmDPFunc dpfunc_dx);
-gnm_float discpfuncinverter (gnm_float p, const gnm_float shape[],
-			     gboolean lower_tail, gboolean log_p,
-			     gnm_float xlow, gnm_float xhigh, gnm_float x0,
-			     GnmPFunc pfunc);
-
-/* ------------------------------------------------------------------------- */
 /* Matrix functions. */
 
 typedef struct {
