@@ -46,6 +46,7 @@ static gboolean ssconvert_list_importers = FALSE;
 static gboolean ssconvert_one_file_per_sheet = FALSE;
 static gboolean ssconvert_recalc = FALSE;
 static gboolean ssconvert_solve = FALSE;
+static char *ssconvert_resize = NULL;
 static char *ssconvert_range = NULL;
 static char *ssconvert_import_encoding = NULL;
 static char *ssconvert_import_id = NULL;
@@ -130,6 +131,13 @@ static const GOptionEntry ssconvert_options [] = {
 		NULL
 	},
 
+	{
+		"resize", 0,
+		G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &ssconvert_resize,
+		N_("Resize to given ROWSxCOLS"),
+		NULL
+	},
+
 
 	/* ---------------------------------------- */
 
@@ -327,7 +335,9 @@ suggest_size (GSList *wbs, int *csuggest, int *rsuggest)
 }
 
 static void
-cb_fixup_name_wb (const char *name, GnmNamedExpr *nexpr, Workbook *wb)
+cb_fixup_name_wb (G_GNUC_UNUSED gconstpointer key,
+		  GnmNamedExpr *nexpr,
+		  Workbook *wb)
 {
 	GnmParsePos newpos = nexpr->pos;
 
@@ -745,6 +755,28 @@ convert (char const *inarg, char const *outarg, char const *mergeargs[],
 			       wbv);
 	}
 
+	if (ssconvert_resize) {
+		int rows, cols;
+		if (sscanf (ssconvert_resize, "%dx%d", &rows, &cols) == 2) {
+			int n;
+
+			g_printerr ("Resizing to %dx%d\n", rows, cols);
+			for (n = workbook_sheet_count (wb) - 1;
+			     n >= 0;
+			     n--) {
+				gboolean err;
+				Sheet *sheet = workbook_sheet_by_index (wb, n);
+				GOUndo *undo =
+					gnm_sheet_resize (sheet, cols, rows,
+							  NULL, &err);
+				if (err)
+					g_printerr ("Resizing of sheet %s failed\n",
+						    sheet->name_unquoted);
+				g_object_unref (undo);
+			}
+		}
+	}
+
 	if (ssconvert_recalc)
 		workbook_recalc_all (wb);
 	gnm_app_recalc ();
