@@ -5,6 +5,8 @@ use File::Basename qw(fileparse);
 use Config;
 use XML::Parser;
 
+$| = 1;
+
 @GnumericTest::ISA = qw (Exporter);
 @GnumericTest::EXPORT = qw(test_sheet_calc test_valgrind
                            test_importer test_exporter test_roundtrip
@@ -15,19 +17,22 @@ use XML::Parser;
 
 use vars qw($topsrc $top_builddir $samples $PERL $verbose);
 use vars qw($ssconvert $ssindex $sstest);
+use vars qw($normalize_gnumeric);
+
+$PERL = $Config{'perlpath'};
+$PERL .= $Config{'_exe'} if $^O ne 'VMS' && $PERL !~ m/$Config{'_exe'}$/i;
 
 $topsrc = $0;
 $topsrc =~ s|/[^/]+$|/..|;
 $topsrc =~ s|/test/\.\.$||;
 
 $top_builddir = "..";
-$samples = "$topsrc/samples";
+$samples = "$topsrc/samples"; $samples =~ s{^\./}{};
 $ssconvert = "$top_builddir/src/ssconvert";
 $ssindex = "$top_builddir/src/ssindex";
 $sstest = "$top_builddir/src/sstest";
+$normalize_gnumeric = "$PERL $topsrc/test/normalize-gnumeric";
 $verbose = 0;
-$PERL = $Config{'perlpath'};
-$PERL .= $Config{'_exe'} if $^O ne 'VMS' && $PERL !~ m/$Config{'_exe'}$/i;
 
 # -----------------------------------------------------------------------------
 
@@ -260,7 +265,7 @@ sub test_importer {
     my $code = system ("$ssconvert '$file' '$tmp' 2>&1 | sed -e 's/^/| /'");
     &system_failure ($ssconvert, $code) if $code;
 
-    my $htxt = `zcat -f '$tmp' | $PERL normalize-gnumeric | sha1sum`;
+    my $htxt = `zcat -f '$tmp' | $normalize_gnumeric | sha1sum`;
     my $newsha1 = lc substr ($htxt, 0, 40);
     die "SHA-1 failure\n" unless $newsha1 =~ /^[0-9a-f]{40}$/;
 
@@ -355,39 +360,66 @@ sub test_exporter {
 # -----------------------------------------------------------------------------
 
 sub test_roundtrip {
-    my ($file,$format,$newext) = @_;
+    my ($file,%named_args) = @_;
 
     &report_skip ("file $file does not exist") unless -r $file;
 
+    my $format = $named_args{'format'};
+    my $newext = $named_args{'ext'};
+    my $resize = $named_args{'resize'};
+    my $ignore_failure = $named_args{'ignore_failure'};
+
+    my $filter1 = $named_args{'filter1'} || $named_args{'filter'} || 'cat';
+    my $filter2 = $named_args{'filter2'} || $named_args{'filter'} || 'cat';
+
     my $tmp = fileparse ($file);
     $tmp =~ s/\.([a-zA-Z0-9]+)$// or die "Must have extension for roundtrip test.";
     my $ext = $1;
     my $code;
     my $keep = 0;
 
+    my $file_resized = $file;
+    if ($resize) {
+	$file_resized =~ s{^.*/}{};
+	$file_resized =~ s/(\.gnumeric)$/-resize$1/;
+	my $cmd = "$ssconvert --resize $resize '$file' '$file_resized'";
+	print STDERR "# $cmd\n" if $verbose;
+	$code = system ("$cmd 2>&1 | sed -e 's/^/| /'");
+	&system_failure ($ssconvert, $code) if $code;
+	&junkfile ($file_resized) unless $keep;
+    }
+    
     my $tmp1 = "$tmp.$newext";
     &junkfile ($tmp1) unless $keep;
-    $code = system ("$ssconvert -T $format '$file' '$tmp1' 2>&1 | sed -e 's/^/| /'");
-    &system_failure ($ssconvert, $code) if $code;
+    {
+	my $cmd = "$ssconvert -T $format '$file_resized' '$tmp1'";
+	print "# $cmd\n" if $verbose;
+	my $code = system ("$cmd 2>&1 | sed -e 's/^/| /'");
+	&system_failure ($ssconvert, $code) if $code;
+    }
 
     my $tmp2 = "$tmp-new.$ext";
     &junkfile ($tmp2) unless $keep;
-    $code = system ("$ssconvert '$tmp1' '$tmp2' 2>&1 | sed -e 's/^/| /'");
-    &system_failure ($ssconvert, $code) if $code;
+    {
+	my $cmd = "$ssconvert '$tmp1' '$tmp2'";
+	print "# $cmd\n" if $verbose;
+	my $code = system ("$cmd 2>&1 | sed -e 's/^/| /'");
+	&system_failure ($ssconvert, $code) if $code;
+    }
 
     my $tmp_xml = "$tmp.xml";
     &junkfile ($tmp_xml) unless $keep;
-    $code = system ("zcat -f '$file' >'$tmp_xml'");
+    $code = system ("zcat -f '$file_resized' | $normalize_gnumeric | $filter1 >'$tmp_xml'");
     &system_failure ('zcat', $code) if $code;
 
     my $tmp2_xml = "$tmp-new.xml";
     &junkfile ($tmp2_xml) unless $keep;
-    $code = system ("zcat -f '$tmp2' >'$tmp2_xml'");
+    # print STDERR "zcat -f '$tmp2' | $normalize_gnumeric | $filter2 >'$tmp2_xml'\n";
+    $code = system ("zcat -f '$tmp2' | $normalize_gnumeric | $filter2 >'$tmp2_xml'");
     &system_failure ('zcat', $code) if $code;
 
     $code = system ('diff', '-u', $tmp_xml, $tmp2_xml);
-    # Ignore failures for now.
-    # &system_failure ('diff', $code) if $code;
+    &system_failure ('diff', $code) if $code && !$ignore_failure;
 
     print STDERR "Pass\n";
 }
@@ -541,9 +573,13 @@ delete $ENV{'G_SLICE'};
 $ENV{'G_DEBUG'} = 'fatal_criticals';
 
 delete $ENV{'LANG'};
+delete $ENV{'LANGUAGE'};
 foreach (keys %ENV) { delete $ENV{$_} if /^LC_/; }
 $ENV{'LC_ALL'} = 'C';
 
+# libgsf listens for this
+delete $ENV{'WINDOWS_LANGUAGE'};
+
 if (@ARGV && $ARGV[0] eq '--verbose') {
     $verbose = 1;
     scalar shift @ARGV;
